<?php

namespace Drop\Import\Console\Command\Import;

use Magento\Framework\App\ObjectManagerFactory;
use Drop\Import\Console\Command\AbstractImportCommand;
use Drop\Import\Model\Attribute;
use Magento\Framework\Exception\NoSuchEntityException;
use Symfony\Component\Console\Input\InputOption;

class GroupPrices extends Products
{

    const CUSTOMER_GROUP_LOGGED_IN_ID = 1;
    const TIER_PRICE_DEFAULT_QTY = 1;
    const CLEAN_PARAMETER = 'clean';

    /**
     * Category Helper: \Drop\Import\Helper\System\Product
     * @var type
     */
    private $helper;
    /**
     * Attribute
     * @var type
     */
    private $attribute;
    /**
     * \Magento\Catalog\Api\ProductRepositoryInterface
     * @var type
     */
    private $productRepository;

    private $state;

    public function __construct(
        ObjectManagerFactory $objectManagerFactory,
        \Magento\Framework\File\Csv $csvProcessor,
        \Drop\Import\Logger\Logger $logger,
        \Magento\Framework\App\State $state,
        \Drop\Import\Helper\System\Product $helper,
        Attribute $attribute,
        \Magento\Catalog\Api\ProductRepositoryInterface\Proxy $productRepository
    )
    {
        $this->helper = $helper;
        $this->attribute = $attribute;
        $this->productRepository = $productRepository;
        $this->state = $state;
        parent::__construct($objectManagerFactory, $csvProcessor, $logger, $state);
    }

    protected function configure()
    {

        $options = [
            new InputOption(
                self::CLEAN_PARAMETER,
                null,
                InputOption::VALUE_NONE,
                'clean special prices'
            )
        ];

        $this->setName('drop:import:group-prices')
            ->setDescription('Import Products Prices by group')
            ->setDefinition($options);

        $this->setEntityCode($this->helper->getEntityCode());
        $this->setBehavior($this->helper->getBehavior());
        $this->setValidationStrategy($this->helper->getValidationStrategy());
        $this->setAllowedErrorCount($this->helper->getAllowedErrorCount());
        $this->setIgnoreDuplicates($this->helper->getIgnoreDuplicates());
        parent::configure();
    }

    /**
     * SAMPLES:
     * $products [] = array(
     * 'sku' => "SIMPLE-RED-MIDDLE",
     * 'price' => '1.0000',
     * );
     * @return array
     */
    protected function getEntities()
    {
        $optionArgument = $this->getOptionsArgument();
        $importMethod = $this->helper->getImportMethod();
        if ($importMethod != \Drop\Import\Model\Config\Source\ImportMethod::IMPORT_METHOD_LOCAL_FILE_PATH) {
            //Download file
            die('TODO');
        }

        $fileName = $this->helper->getGroupPriceImportPathUrl();
        $csvIterationObject = $this->readCSV(BP . '/' . $fileName);
        $header = $this->getCsvHeader();
        $languages = $this->getLanguages();
        $products = [];
        $i = 0;
        // Controllo codifica file
        foreach($header as $headValue) {
            if (mb_detect_encoding($headValue, mb_detect_order(), TRUE) != "ASCII") {
                echo "-ATTENTIONE- controlla la codifica del file csv, dev'essere ASCII";
            }
        }

        // FIX: Magento restituisce area code not set durante il salvataggio del prodotto anche se è già stata settata
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $state = $objectManager->get('Magento\Framework\App\State');
        $state->setAreaCode('global');

        foreach ($csvIterationObject as $row) {
            if(empty($row['sku'])) {
                echo PHP_EOL . "SKU was not found. Skipping.";
                continue;
            }

            $row['sku'] = strtolower($row['sku']);
            $originalSku = $row['sku'];
            if (strrpos($row['sku'], '-') !== false) {
                $row['sku'] = substr($row['sku'], 0, strrpos($row['sku'], '-'));
            }

            try {
                $_product = $this->productRepository->get($row['sku']);
            } catch (NoSuchEntityException $e) {
                echo PHP_EOL . strtoupper($row['sku']) . ' NoSuchEntityException on Other Attributes: ' . $e->getMessage() . '. Skipping.';
                continue;
            } catch (\Exception $e) {
                echo PHP_EOL . strtoupper($row['sku']) . ' Exception on Other Attributes: ' . $e->getMessage() . '. Skipping.';
                continue;
            }

            if($_product && !empty($simpleAssociated = $this->getSimpleAssociated($_product))) {
                foreach($simpleAssociated as $simpleProduct) {
//                    $alreadySetted = false;
//                    foreach($simpleProduct->getTierPrices() as $tier) {
//                        if( $tier->getCustomerGroupId() == self::CUSTOMER_GROUP_LOGGED_IN_ID &&
//                            $tier->getQty() == self::TIER_PRICE_DEFAULT_QTY &&
//                            $tier->getValue() == $row['saldi_pto_pzzo']) {
//                            $alreadySetted = true;
//                        }
//                    }
//                    if($alreadySetted) {
//                        continue;
//                    }

                    if(strpos($simpleProduct->getSku(), $originalSku) === false) {
                        continue;
                    }

                    try {
                        // Costretto a ricare il prodotto tramite OBJ Manager altrimenti durante il salvataggio restituisce: Lock wait timeout exceeded
                        $productRepository = $objectManager->get('Magento\Catalog\Model\ProductRepository');
                        $loadedSimpleProduct = $productRepository->getById($simpleProduct->getId());
                        if (isset($optionArgument[self::CLEAN_PARAMETER])) {
                            $loadedSimpleProduct->setTierPrice([])->save();
                            echo PHP_EOL . 'RESETetted group price for: ' . $simpleProduct->getSku();
                        } else {
                            $loadedSimpleProduct
                                ->setTierPrice([[
                                    'website_id' => 1,
                                    'cust_group' => self::CUSTOMER_GROUP_LOGGED_IN_ID,
                                    'price_qty' => self::TIER_PRICE_DEFAULT_QTY,
                                    'price' => $row['saldi_pto_pzzo'],
                                ]])->save();
                            echo PHP_EOL . 'Setted group price for: ' . $simpleProduct->getSku();
                        }
                    } catch (\Exception $e) {
                        echo PHP_EOL . 'Exception for ' . $simpleProduct->getSku() . ': ' . $e->getMessage();
                    }
                }
            }
        }

        if(!count($products)) {
            throw new \Exception('There are no product to import. Exit');
        }
/*
        echo "<br/><pre>";
        print_r($products);
        echo "<pre><br/>";
        die(__METHOD__ . '1');
*/
        return $products;
    }

    /**
     * @param $simpleSku
     * @param $languages
     * @param $row
     * @param $i
     * @return array
     */
    public function getLanguagesVariables($simpleSku, $languages, $row, $i)
    {
        $products = [];
        $langCounter = 0;
        foreach ($languages as $lang) {
            if ($lang == self::DEFAULT_STORE) {
                continue;
            }
            $i++;
            $products[$i] = [
                'sku'             => $simpleSku,
                'store_view_code' => $lang,
                'price'           => $this->helper->cleanPriceString($row['prezzo_' . $lang])
            ];
            $langCounter++;
        }

        return $products;
    }

    /**
     * @param $row
     * @param $i
     * @return mixed
     */
    public function getUpdatedConfigurableVisibility($row, $i)
    {
        $products = [
            'sku'             => $row['sku'],
            'visibility'      => 'Catalog, Search'
        ];

        return $products;
    }
}
